/***************************************************************************//**
* \file cy_ble_hal_pvt.c
* \version `$CY_MAJOR_VERSION`.`$CY_MINOR_VERSION`
*
* \brief
*  This file contains the source code for the HAL section of the BLE Middleware.
*
********************************************************************************
* \copyright
* Copyright 2017-2018, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/


#include "cy_ble_hal_pvt.h"
#include <string.h>
/* C binding of definitions if building with C++ compiler */
#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


    
/*******************************************************************************
* Function Name: Cy_BLE_HAL_RadioCalibrationWrite
****************************************************************************//**
*
*  Write radio calibration data to SFLASH
*
*  \param data:   Pointer to the buffer containing the data to be stored.
*  \param length: The length of the data in bytes.
*
*  \return
*   CY_BLE_SUCCESS                     A successful write
*   CY_BLE_ERROR_INVALID_PARAMETER     At least one of the input parameters is invalid
*   CY_BLE_ERROR_FLASH_WRITE           Error in flash Write
*
*******************************************************************************/
cy_en_ble_api_result_t Cy_BLE_HAL_RadioCalibrationWrite(const void *data, uint32_t length)
{
    cy_en_ble_api_result_t rc;

    if((data != NULL) &&
       (length <= (CY_FLASH_SIZEOF_ROW - sizeof(cy_stc_ble_gap_bd_addr_t))))
    {
//        rc = Cy_BLE_HAL_NvramWrite(data, (uint8_t *)CY_BLE_SFLASH_BLE_RADIO_CALL_ADDRESS,
//                                   length, CY_BLE_FLASH_BLOCKING_MODE);
    }
    else
    {
        rc = CY_BLE_ERROR_INVALID_PARAMETER;
    }
    return(rc);
}


/*******************************************************************************
* Function Name: Cy_BLE_HAL_RadioCalibrationRead
****************************************************************************//**
*
*  Read radio calibration data from SFLASH
*
*  \param data:   Pointer to the buffer containing the data to be read from SFASH.
*  \param length: The length of the data in bytes.
*
*  \return
*   CY_BLE_SUCCESS                     A successful read
*   CY_BLE_ERROR_INVALID_PARAMETER     At least one of the input parameters is invalid
*
*******************************************************************************/
cy_en_ble_api_result_t Cy_BLE_HAL_RadioCalibrationRead(void *data, uint32_t length)
{
    cy_en_ble_api_result_t rc = CY_BLE_SUCCESS;
    
    if((data != NULL) && 
       (length <= (CY_FLASH_SIZEOF_ROW - sizeof(cy_stc_ble_gap_bd_addr_t))))
    {
        memcpy(data, (void *)CY_BLE_SFLASH_BLE_RADIO_CALL_ADDRESS, length);
    }
    else
    {
        rc = CY_BLE_ERROR_INVALID_PARAMETER;
    }
    return(rc);
}




/*******************************************************************************
* Function Name: Cy_BLE_HAL_GetIcPackageType
****************************************************************************//**
*
*  This function returns package type
*
*  return:
*   0 - BGA_116 package
*   1 - CSP_104 package
*   2 - BGA_124 package
*   3 - QFN_56 package
*
*******************************************************************************/
uint32_t Cy_BLE_HAL_GetIcPackageType(void)
{
    /* Constants used in this function */
    const uint8_t PACKAGE_116_PINS = 116u;   //116BGA
    const uint8_t PACKAGE_104_PINS = 104u;   //104MCSP
    const uint8_t PACKAGE_124_PINS = 124u;   //124BGA

#if defined(CY_GPIO_PIN_COUNT) 
    uint8_t pinCount = CY_GPIO_PIN_COUNT;
#else
    //uint8_t pinCount = 124;        //WEIY, select 124-BGA
    //uint8_t pinCount = 116;        //WEIY, select 116-BGA
    uint8_t pinCount = 104;        //WEIY, select 104-MCSP
#endif /* defined(CY_GPIO_PIN_COUNT) */
            
    cy_en_ble_ic_package_type_t ret;

    if(pinCount == PACKAGE_116_PINS)       //04-24-2019, WEIY
    {
        ret = CY_BLE_IC_PACKAGE_TYPE_BGA_116;
    }
    else if( pinCount == PACKAGE_104_PINS ) 
    {
        ret = CY_BLE_IC_PACKAGE_TYPE_CSP_104;
    }
    else if( pinCount == PACKAGE_124_PINS)
    {
        ret = CY_BLE_IC_PACKAGE_TYPE_BGA_124;
    }
    else
    {
        ret = CY_BLE_IC_PACKAGE_TYPE_QFN_68;
    }
    return((uint32_t)ret);
}


/*******************************************************************************
* Function Name: Cy_BLE_HAL_RadioGetAbsVbat
****************************************************************************//**
*
* This function returns absolute value of voltage.
*
* input param: raw - pointer to the 16 bit result from MXD registers,
* output param: vbat - pointer to the calculated voltage value in mV.
* 
*******************************************************************************/
void Cy_BLE_HAL_RadioGetAbsVbat(const uint16_t *raw, uint16_t *vbat)
{
     *vbat = ((CY_BLE_RADIO_VOLTAGE_MONITOR_SLOPE * (*raw)) - CY_BLE_RADIO_VOLTAGE_MONITOR_OFFSET) / 
               CY_BLE_RADIO_VOLTAGE_MONITOR_SLOPE_MULTIPLIER;
}


/*******************************************************************************
* Function Name: Cy_BLE_HAL_RadioGetAbsTemp
****************************************************************************//**
*
* This function returns absolute value temperature.
*
* input param: raw - pointer to the the 16 bit result from MXD registers,
* output param: temp - pointer to the int16_t calculated temperature value in C.
* 
*******************************************************************************/
void Cy_BLE_HAL_RadioGetAbsTemp(const uint16_t *raw, int16_t *temp)
{
    *temp = CY_BLE_RADIO_TEMP_MONITOR_OFFSET - 
            ((CY_BLE_RADIO_TEMP_MONITOR_SLOPE * (int16_t)(*raw)) / CY_BLE_RADIO_TEMP_MONITOR_SLOPE_MULTIPLIER);
}


/*******************************************************************************
* Function Name: Cy_BLE_HAL_GetUniqueId
****************************************************************************//**
* Interface to Cy_SysLib_GetUniqueId function.
*
* This function returns the silicon unique ID.
* The ID includes Die lot[3]#, Die Wafer#, Die X, Die Y, Die Sort#, Die Minor
* and Die Year.
*
* \return  A combined 64-bit unique ID.
*          [63:57] - DIE_YEAR
*          [56:56] - DIE_MINOR
*          [55:48] - DIE_SORT
*          [47:40] - DIE_Y
*          [39:32] - DIE_X
*          [31:24] - DIE_WAFER
*          [23:16] - DIE_LOT[2]
*          [15: 8] - DIE_LOT[1]
*          [ 7: 0] - DIE_LOT[0]
*
*******************************************************************************/
uint64_t Cy_BLE_HAL_GetUniqueId(void)
{
    return(Cy_SysLib_GetUniqueId());
}


/*******************************************************************************
* Function Name: Cy_BLE_HAL_GetSiliconId
****************************************************************************//**
*
* This function returns the Silicon ID of the device.
*
* \return  Silicon ID
*
*******************************************************************************/
uint16_t Cy_BLE_HAL_GetSiliconId(void)
{
    return(SFLASH->SILICON_ID);
}

/*******************************************************************************
* Function Name: Cy_BLE_HAL_IsDefinedTx5dbm
****************************************************************************//**
*
*  This function returns true if Tx 5dbm mode is defined
*
* return:
*  true  - Tx 5dbm mode is defined
*  false - Tx 5dbm mode is not defined
*
*******************************************************************************/
bool Cy_BLE_HAL_IsDefinedTx5dbm(void)
{
    bool ret = false;

    #if (defined (CY_BLE_ENABLE_TX_5DBM) && (CY_BLE_ENABLE_TX_5DBM != 0u))
        ret = true;
    #endif /* ((defined (CY_BLE_ENABLE_TX_5DBM)) && (CY_BLE_ENABLE_TX_5DBM != 0u)) */

    return(ret);
}



/*******************************************************************************
* Function Name: Cy_BLE_HAL_SimoBuckSetVoltage2
****************************************************************************//**
*
*  Interface to Cy_SysPm_SimoBuckSetVoltage2 function with disable the 200 uS
*  delay after setting a higher voltage.
*
*******************************************************************************/
void Cy_BLE_HAL_SimoBuckSetVoltage2(cy_en_syspm_simo_buck_voltage2_t voltage)
{
    Cy_SysPm_SimoBuckSetVoltage2(voltage, false);
}
#ifdef __cplusplus
}
#endif /* __cplusplus */

/* [] END OF FILE */
